@@ -1,13 +1,19 @@
 package toxiproxy_test
 
 import (
-	"io/ioutil"
+	"bytes"
+	"flag"
+	"io"
 	"net/http"
+	"os"
 	"testing"
 	"time"
 
-	"github.com/Shopify/toxiproxy"
-	tclient "github.com/Shopify/toxiproxy/client"
+	"github.com/prometheus/client_golang/prometheus"
+	"github.com/rs/zerolog"
+
+	"github.com/Shopify/toxiproxy/v2"
+	tclient "github.com/Shopify/toxiproxy/v2/client"
 )
 
 var testServer *toxiproxy.ApiServer
@@ -15,11 +21,20 @@ var testServer *toxiproxy.ApiServer
 var client = tclient.NewClient("http://127.0.0.1:8475")
 
 func WithServer(t *testing.T, f func(string)) {
+	log := zerolog.Nop()
+	if flag.Lookup("test.v").DefValue == "true" {
+		log = zerolog.New(os.Stdout).With().Caller().Timestamp().Logger()
+	}
+
 	// Make sure only one server is running at a time. Apparently there's no clean
 	// way to shut it down between each test run.
 	if testServer == nil {
-		testServer = toxiproxy.NewServer()
-		go testServer.Listen("localhost", "8475")
+		testServer = toxiproxy.NewServer(
+			toxiproxy.NewMetricsContainer(prometheus.NewRegistry()),
+			log,
+		)
+
+		go testServer.Listen("localhost:8475")
 
 		// Allow server to start. There's no clean way to know when it listens.
 		time.Sleep(50 * time.Millisecond)
@@ -35,6 +50,67 @@ func WithServer(t *testing.T, f func(string)) {
 	f("http://localhost:8475")
 }
 
+func TestRequestId(t *testing.T) {
+	WithServer(t, func(addr string) {
+		client := http.Client{}
+
+		req, _ := http.NewRequest("GET", "http://localhost:8475/version", nil)
+		req.Header.Add("User-Agent", "curl")
+
+		resp, err := client.Do(req)
+		if err != nil {
+			t.Fatalf("Does not expect errors from client: %+v", err)
+		}
+		defer resp.Body.Close()
+
+		if _, ok := resp.Header["X-Toxiproxy-Request-Id"]; !ok {
+			t.Fatalf("Expect http response with header X-Toxiproxy-Request-Id, got %+v", resp.Header)
+		}
+	})
+}
+
+func TestBrowserGets403(t *testing.T) {
+	WithServer(t, func(addr string) {
+		client := http.Client{}
+
+		req, _ := http.NewRequest("GET", "http://localhost:8475/proxies", nil)
+		req.Header.Add(
+			"User-Agent",
+			"Mozilla/5.0 (Linux; Android 4.4.2); Nexus 5 Build/KOT49H) AppleWebKit/537.36"+
+				"(KHTML, like Gecko) Chrome/33.0.1750.117 Mobile Safari/537.36 OPR/20.0.1396.72047",
+		)
+
+		resp, err := client.Do(req)
+		if err != nil {
+			t.Fatalf("Does not expect errors from client: %v", err)
+		}
+		defer resp.Body.Close()
+
+		if resp.StatusCode != 403 {
+			t.Fatal("Browser-like UserAgent was not denied access to Toxiproxy")
+		}
+	})
+}
+
+func TestNonBrowserGets200(t *testing.T) {
+	WithServer(t, func(addr string) {
+		client := http.Client{}
+
+		req, _ := http.NewRequest("GET", "http://localhost:8475/proxies", nil)
+		req.Header.Add("User-Agent", "Wget/2.1")
+
+		resp, err := client.Do(req)
+		if err != nil {
+			t.Fatalf("Does not expect errors from client: %v", err)
+		}
+		defer resp.Body.Close()
+
+		if resp.StatusCode == 403 {
+			t.Fatal("Non-Browser-like UserAgent was denied access to Toxiproxy")
+		}
+	})
+}
+
 func TestIndexWithNoProxies(t *testing.T) {
 	WithServer(t, func(addr string) {
 		client := tclient.NewClient(addr)
@@ -52,10 +128,12 @@ func TestIndexWithNoProxies(t *testing.T) {
 func TestCreateProxyBlankName(t *testing.T) {
 	WithServer(t, func(addr string) {
 		_, err := client.CreateProxy("", "", "")
+
+		expected := "Create: HTTP 400: missing required field: name"
 		if err == nil {
-			t.Fatal("Expected error creating proxy, got nil")
-		} else if err.Error() != "Create: HTTP 400: missing required field: name" {
-			t.Fatal("Expected different error creating proxy:", err)
+			t.Error("Expected error creating proxy, got nil")
+		} else if err.Error() != expected {
+			t.Errorf("Expected error `%s',\n\tgot: `%s'", expected, err)
 		}
 	})
 }
@@ -64,9 +142,300 @@ func TestCreateProxyBlankUpstream(t *testing.T) {
 	WithServer(t, func(addr string) {
 		_, err := client.CreateProxy("test", "", "")
 		if err == nil {
-			t.Fatal("Expected error creating proxy, got nil")
+			t.Error("Expected error creating proxy, got nil")
 		} else if err.Error() != "Create: HTTP 400: missing required field: upstream" {
-			t.Fatal("Expected different error creating proxy:", err)
+			t.Error("Expected different error creating proxy:", err)
+		}
+	})
+}
+
+func TestPopulateProxy(t *testing.T) {
+	WithServer(t, func(addr string) {
+		testProxies, err := client.Populate([]tclient.Proxy{
+			{
+				Name:     "one",
+				Listen:   "localhost:7070",
+				Upstream: "localhost:7171",
+				Enabled:  true,
+			},
+			{
+				Name:     "two",
+				Listen:   "localhost:7373",
+				Upstream: "localhost:7474",
+				Enabled:  true,
+			},
+		})
+
+		if err != nil {
+			t.Fatal("Unable to populate:", err)
+		}
+
+		if len(testProxies) != 2 {
+			t.Fatalf("Wrong number of proxies returned: %d != 2", len(testProxies))
+		}
+
+		if testProxies[0].Name != "one" || testProxies[1].Name != "two" {
+			t.Fatalf("Wrong proxy names returned: %s, %s", testProxies[0].Name, testProxies[1].Name)
+		}
+
+		for _, p := range testProxies {
+			AssertProxyUp(t, p.Listen, true)
+		}
+	})
+}
+
+func TestPopulateDefaultEnabled(t *testing.T) {
+	WithServer(t, func(addr string) {
+		request := []byte(
+			`[{"name": "test", "listen": "localhost:7070", "upstream": "localhost:7171"}]`,
+		)
+
+		resp, err := http.Post(addr+"/populate", "application/json", bytes.NewReader(request))
+		if err != nil {
+			t.Fatal("Failed to send POST to /populate:", err)
+		}
+		defer resp.Body.Close()
+
+		if resp.StatusCode != http.StatusCreated {
+			message, _ := io.ReadAll(resp.Body)
+			t.Fatalf("Failed to populate proxy list: HTTP %s\n%s", resp.Status, string(message))
+		}
+
+		proxies, err := client.Proxies()
+		if err != nil {
+			t.Fatal(err)
+		} else if len(proxies) != 1 {
+			t.Fatalf("Wrong number of proxies created: %d != 1", len(proxies))
+		} else if _, ok := proxies["test"]; !ok {
+			t.Fatalf("Wrong proxy name returned")
+		}
+
+		for _, p := range proxies {
+			AssertProxyUp(t, p.Listen, true)
+		}
+	})
+}
+
+func TestPopulateDisabledProxy(t *testing.T) {
+	WithServer(t, func(addr string) {
+		testProxies, err := client.Populate([]tclient.Proxy{
+			{
+				Name:     "one",
+				Listen:   "localhost:7070",
+				Upstream: "localhost:7171",
+				Enabled:  false,
+			},
+			{
+				Name:     "two",
+				Listen:   "localhost:7373",
+				Upstream: "localhost:7474",
+				Enabled:  true,
+			},
+		})
+
+		if err != nil {
+			t.Fatal("Unable to populate:", err)
+		}
+
+		if len(testProxies) != 2 {
+			t.Fatalf("Wrong number of proxies returned: %d != 2", len(testProxies))
+		}
+
+		if testProxies[0].Name != "one" || testProxies[1].Name != "two" {
+			t.Fatalf("Wrong proxy names returned: %s, %s", testProxies[0].Name, testProxies[1].Name)
+		}
+
+		AssertProxyUp(t, "localhost:7070", false)
+		AssertProxyUp(t, "localhost:7373", true)
+	})
+}
+
+func TestPopulateExistingProxy(t *testing.T) {
+	WithServer(t, func(addr string) {
+		testProxy, err := client.CreateProxy("one", "localhost:7070", "localhost:7171")
+		if err != nil {
+			t.Fatal("Unable to create proxy:", err)
+		}
+
+		_, err = client.CreateProxy("two", "localhost:7373", "localhost:7474")
+		if err != nil {
+			t.Fatal("Unable to create proxy:", err)
+		}
+
+		// Create a toxic so we can make sure the proxy wasn't replaced
+		_, err = testProxy.AddToxic("", "latency", "downstream", 1, nil)
+		if err != nil {
+			t.Fatal("Unable to create toxic:", err)
+		}
+
+		testProxies, err := client.Populate([]tclient.Proxy{
+			{
+				Name:     "one",
+				Listen:   "localhost:7070", // intentional: this should be resolved to 127.0.0.1:7070
+				Upstream: "localhost:7171",
+				Enabled:  true,
+			},
+			{
+				Name:     "two",
+				Listen:   "127.0.0.1:7575",
+				Upstream: "localhost:7676",
+				Enabled:  true,
+			},
+		})
+
+		if err != nil {
+			t.Fatal("Unable to populate:", err)
+		}
+
+		if len(testProxies) != 2 {
+			t.Fatalf("Wrong number of proxies returned: %d != 2", len(testProxies))
+		}
+
+		if testProxies[0].Name != "one" || testProxies[1].Name != "two" {
+			t.Fatalf("Wrong proxy names returned: %s, %s", testProxies[0].Name, testProxies[1].Name)
+		}
+
+		if testProxies[0].Listen != "127.0.0.1:7070" ||
+			testProxies[1].Listen != "127.0.0.1:7575" {
+			t.Fatalf("Wrong proxy listen addresses returned: %s, %s",
+				testProxies[0].Listen, testProxies[1].Listen,
+			)
+		}
+
+		toxics, err := testProxy.Toxics()
+		if err != nil {
+			t.Fatal("Unable to get toxics:", err)
+		}
+		if len(toxics) != 1 || toxics[0].Type != "latency" {
+			t.Fatalf("Populate did not preseve existing proxy. (%d toxics)", len(toxics))
+		}
+
+		for _, p := range testProxies {
+			AssertProxyUp(t, p.Listen, true)
+		}
+	})
+}
+
+func TestPopulateWithBadName(t *testing.T) {
+	WithServer(t, func(addr string) {
+		testProxies, err := client.Populate([]tclient.Proxy{
+			{
+				Name:     "one",
+				Listen:   "localhost:7070",
+				Upstream: "localhost:7171",
+				Enabled:  true,
+			},
+			{
+				Name:    "",
+				Listen:  "",
+				Enabled: true,
+			},
+		})
+
+		if err == nil {
+			t.Fatal("Expected Populate to fail.")
+		}
+
+		expected := "Populate: HTTP 400: missing required field: name at proxy 2"
+		if err.Error() != expected {
+			t.Fatalf("Expected error `%s',\n\tgot: `%s'", expected, err)
+		}
+
+		if len(testProxies) != 0 {
+			t.Fatalf("Wrong number of proxies returned: %d != 0", len(testProxies))
+		}
+
+		proxies, err := client.Proxies()
+		if err != nil {
+			t.Fatal(err)
+		} else if len(proxies) != 0 {
+			t.Fatalf("Expected no proxies to be created: %d != 0", len(proxies))
+		}
+	})
+}
+
+func TestPopulateProxyWithBadDataShouldReturnError(t *testing.T) {
+	WithServer(t, func(addr string) {
+		testProxies, err := client.Populate([]tclient.Proxy{
+			{
+				Name:     "one",
+				Listen:   "localhost:7070",
+				Upstream: "localhost:7171",
+				Enabled:  true,
+			},
+			{
+				Name:     "two",
+				Listen:   "local373",
+				Upstream: "localhost:7474",
+				Enabled:  true,
+			},
+			{
+				Name:     "three",
+				Listen:   "localhost:7575",
+				Upstream: "localhost:7676",
+				Enabled:  true,
+			},
+		})
+
+		if err == nil {
+			t.Fatal("Expected Populate to fail.")
+		}
+
+		if len(testProxies) != 0 {
+			t.Fatalf("Expected Proxies to be empty, got %v", testProxies)
+		}
+
+		proxies, err := client.Proxies()
+		if err != nil {
+			t.Fatalf("Expected no error, got: %v", err)
+		}
+
+		if len(proxies) != 1 {
+			t.Fatalf("Wrong number of proxies returned: %d != %d", len(proxies), 1)
+		}
+
+		if _, ok := proxies["one"]; !ok {
+			t.Fatal("Proxy `one' was not created!")
+		}
+
+		for _, p := range testProxies {
+			AssertProxyUp(t, p.Listen, true)
+		}
+
+		for _, p := range proxies {
+			if p.Name == "two" || p.Name == "three" {
+				t.Fatalf("Proxy %s exists, populate did not fail correctly.", p.Name)
+			}
+		}
+	})
+}
+
+func TestPopulateAddToxic(t *testing.T) {
+	WithServer(t, func(addr string) {
+		testProxies, err := client.Populate([]tclient.Proxy{
+			{
+				Name:     "one",
+				Listen:   "localhost:7070",
+				Upstream: "localhost:7171",
+				Enabled:  true,
+			},
+		})
+
+		if err != nil {
+			t.Fatal("Unable to populate:", err)
+		}
+
+		if len(testProxies) != 1 {
+			t.Fatalf("Wrong number of proxies returned: %d != %d", len(testProxies), 1)
+		}
+
+		if testProxies[0].Name != "one" {
+			t.Fatalf("Wrong proxy name returned: %s != one", testProxies[0].Name)
+		}
+
+		_, err = testProxies[0].AddToxic("", "latency", "downstream", 1, nil)
+		if err != nil {
+			t.Fatal("Failed to AddToxic.")
 		}
 	})
 }
@@ -90,8 +459,14 @@ func TestListingProxies(t *testing.T) {
 		if !ok {
 			t.Fatal("Expected to see mysql_master proxy in list")
 		}
-		if proxy.Name != "mysql_master" || proxy.Listen != "127.0.0.1:3310" || proxy.Upstream != "localhost:20001" {
-			t.Fatalf("Unexpected proxy metadata: %s, %s, %s", proxy.Name, proxy.Listen, proxy.Upstream)
+		if proxy.Name != "mysql_master" || proxy.Listen != "127.0.0.1:3310" ||
+			proxy.Upstream != "localhost:20001" {
+			t.Fatalf(
+				"Unexpected proxy metadata: %s, %s, %s",
+				proxy.Name,
+				proxy.Listen,
+				proxy.Upstream,
+			)
 		}
 		AssertToxicExists(t, proxy.ActiveToxics, "latency", "", "", false)
 	})
@@ -109,8 +484,16 @@ func TestCreateAndGetProxy(t *testing.T) {
 			t.Fatal("Unable to retriecve proxy:", err)
 		}
 
-		if proxy.Name != "mysql_master" || proxy.Listen != "127.0.0.1:3310" || proxy.Upstream != "localhost:20001" || !proxy.Enabled {
-			t.Fatalf("Unexpected proxy metadata: %s, %s, %s, %v", proxy.Name, proxy.Listen, proxy.Upstream, proxy.Enabled)
+		if proxy.Name != "mysql_master" || proxy.Listen != "127.0.0.1:3310" ||
+			proxy.Upstream != "localhost:20001" ||
+			!proxy.Enabled {
+			t.Fatalf(
+				"Unexpected proxy metadata: %s, %s, %s, %v",
+				proxy.Name,
+				proxy.Listen,
+				proxy.Upstream,
+				proxy.Enabled,
+			)
 		}
 
 		AssertToxicExists(t, proxy.ActiveToxics, "latency", "", "", false)
@@ -135,8 +518,16 @@ func TestCreateProxyWithSave(t *testing.T) {
 			t.Fatal("Unable to retriecve proxy:", err)
 		}
 
-		if proxy.Name != "mysql_master" || proxy.Listen != "127.0.0.1:3310" || proxy.Upstream != "localhost:20001" || !proxy.Enabled {
-			t.Fatalf("Unexpected proxy metadata: %s, %s, %s, %v", proxy.Name, proxy.Listen, proxy.Upstream, proxy.Enabled)
+		if proxy.Name != "mysql_master" || proxy.Listen != "127.0.0.1:3310" ||
+			proxy.Upstream != "localhost:20001" ||
+			!proxy.Enabled {
+			t.Fatalf(
+				"Unexpected proxy metadata: %s, %s, %s, %v",
+				proxy.Name,
+				proxy.Listen,
+				proxy.Upstream,
+				proxy.Enabled,
+			)
 		}
 
 		AssertProxyUp(t, proxy.Listen, true)
@@ -160,8 +551,16 @@ func TestCreateDisabledProxy(t *testing.T) {
 			t.Fatal("Unable to retriecve proxy:", err)
 		}
 
-		if proxy.Name != "mysql_master" || proxy.Listen != "localhost:3310" || proxy.Upstream != "localhost:20001" || proxy.Enabled {
-			t.Fatalf("Unexpected proxy metadata: %s, %s, %s, %v", proxy.Name, proxy.Listen, proxy.Upstream, proxy.Enabled)
+		if proxy.Name != "mysql_master" || proxy.Listen != "localhost:3310" ||
+			proxy.Upstream != "localhost:20001" ||
+			proxy.Enabled {
+			t.Fatalf(
+				"Unexpected proxy metadata: %s, %s, %s, %v",
+				proxy.Name,
+				proxy.Listen,
+				proxy.Upstream,
+				proxy.Enabled,
+			)
 		}
 
 		AssertProxyUp(t, proxy.Listen, false)
@@ -185,8 +584,16 @@ func TestCreateDisabledProxyAndEnable(t *testing.T) {
 			t.Fatal("Unable to retriecve proxy:", err)
 		}
 
-		if proxy.Name != "mysql_master" || proxy.Listen != "localhost:3310" || proxy.Upstream != "localhost:20001" || proxy.Enabled {
-			t.Fatalf("Unexpected proxy metadata: %s, %s, %s, %v", proxy.Name, proxy.Listen, proxy.Upstream, proxy.Enabled)
+		if proxy.Name != "mysql_master" || proxy.Listen != "localhost:3310" ||
+			proxy.Upstream != "localhost:20001" ||
+			proxy.Enabled {
+			t.Fatalf(
+				"Unexpected proxy metadata: %s, %s, %s, %v",
+				proxy.Name,
+				proxy.Listen,
+				proxy.Upstream,
+				proxy.Enabled,
+			)
 		}
 
 		proxy.Enabled = true
@@ -243,11 +650,12 @@ func TestDeleteProxy(t *testing.T) {
 			t.Fatal("Expected proxy to be deleted from list")
 		}
 
+		expected := "Delete: HTTP 404: proxy not found"
 		err = testProxy.Delete()
 		if err == nil {
-			t.Fatal("Proxy did not result in not found.")
-		} else if err.Error() != "Delete: HTTP 404: proxy not found" {
-			t.Fatal("Incorrect error removing proxy:", err)
+			t.Error("Proxy did not result in not found.")
+		} else if err.Error() != expected {
+			t.Errorf("Expected error `%s',\n\tgot: `%s'", expected, err)
 		}
 	})
 }
@@ -259,11 +667,12 @@ func TestCreateProxyPortConflict(t *testing.T) {
 			t.Fatal("Unable to create proxy:", err)
 		}
 
+		expected := "Create: HTTP 500: listen tcp 127.0.0.1:3310: bind: address already in use"
 		_, err = client.CreateProxy("test", "localhost:3310", "localhost:20001")
 		if err == nil {
-			t.Fatal("Proxy did not result in conflict.")
-		} else if err.Error() != "Create: HTTP 500: listen tcp 127.0.0.1:3310: bind: address already in use" {
-			t.Fatal("Incorrect error adding proxy:", err)
+			t.Error("Proxy did not result in conflict.")
+		} else if err.Error() != expected {
+			t.Errorf("Expected error `%s',\n\tgot: `%s'", expected, err)
 		}
 
 		err = testProxy.Delete()
@@ -284,11 +693,12 @@ func TestCreateProxyNameConflict(t *testing.T) {
 			t.Fatal("Unable to create proxy:", err)
 		}
 
+		expected := "Create: HTTP 409: proxy already exists"
 		_, err = client.CreateProxy("mysql_master", "localhost:3311", "localhost:20001")
 		if err == nil {
 			t.Fatal("Proxy did not result in conflict.")
-		} else if err.Error() != "Create: HTTP 409: proxy already exists" {
-			t.Fatal("Incorrect error adding proxy:", err)
+		} else if err.Error() != expected {
+			t.Fatalf("Expected error `%s',\n\tgot: `%s'", expected, err)
 		}
 
 		err = testProxy.Delete()
@@ -390,7 +800,8 @@ func TestAddToxic(t *testing.T) {
 			t.Fatal("Error returning toxics:", err)
 		}
 		toxic := AssertToxicExists(t, toxics, "foobar", "latency", "downstream", true)
-		if toxic.Toxicity != 1.0 || toxic.Attributes["latency"] != 100.0 || toxic.Attributes["jitter"] != 10.0 {
+		if toxic.Toxicity != 1.0 || toxic.Attributes["latency"] != 100.0 ||
+			toxic.Attributes["jitter"] != 10.0 {
 			t.Fatal("Toxic was not read back correctly:", toxic)
 		}
 	})
@@ -419,7 +830,8 @@ func TestAddMultipleToxics(t *testing.T) {
 		}
 		AssertToxicExists(t, toxics, "latency1", "latency", "downstream", true)
 		toxic := AssertToxicExists(t, toxics, "latency2", "latency", "downstream", true)
-		if toxic.Toxicity != 1.0 || toxic.Attributes["latency"] != 0.0 || toxic.Attributes["jitter"] != 0.0 {
+		if toxic.Toxicity != 1.0 || toxic.Attributes["latency"] != 0.0 ||
+			toxic.Attributes["jitter"] != 0.0 {
 			t.Fatal("Toxic was not read back correctly:", toxic)
 		}
 		AssertToxicExists(t, toxics, "latency1", "", "upstream", false)
@@ -451,7 +863,8 @@ func TestAddConflictingToxic(t *testing.T) {
 			t.Fatal("Error returning toxics:", err)
 		}
 		toxic := AssertToxicExists(t, toxics, "foobar", "latency", "downstream", true)
-		if toxic.Toxicity != 1.0 || toxic.Attributes["latency"] != 0.0 || toxic.Attributes["jitter"] != 0.0 {
+		if toxic.Toxicity != 1.0 || toxic.Attributes["latency"] != 0.0 ||
+			toxic.Attributes["jitter"] != 0.0 {
 			t.Fatal("Toxic was not read back correctly:", toxic)
 		}
 		AssertToxicExists(t, toxics, "foobar", "", "upstream", false)
@@ -483,7 +896,8 @@ func TestAddConflictingToxicsMultistream(t *testing.T) {
 		}
 
 		toxic := AssertToxicExists(t, toxics, "foobar", "latency", "upstream", true)
-		if toxic.Toxicity != 1.0 || toxic.Attributes["latency"] != 0.0 || toxic.Attributes["jitter"] != 0.0 {
+		if toxic.Toxicity != 1.0 || toxic.Attributes["latency"] != 0.0 ||
+			toxic.Attributes["jitter"] != 0.0 {
 			t.Fatal("Toxic was not read back correctly:", toxic)
 		}
 		AssertToxicExists(t, toxics, "foobar", "", "downstream", false)
@@ -512,11 +926,13 @@ func TestAddConflictingToxicsMultistreamDefaults(t *testing.T) {
 			t.Fatal("Error returning toxics:", err)
 		}
 		toxic := AssertToxicExists(t, toxics, "latency_upstream", "latency", "upstream", true)
-		if toxic.Toxicity != 1.0 || toxic.Attributes["latency"] != 0.0 || toxic.Attributes["jitter"] != 0.0 {
+		if toxic.Toxicity != 1.0 || toxic.Attributes["latency"] != 0.0 ||
+			toxic.Attributes["jitter"] != 0.0 {
 			t.Fatal("Toxic was not read back correctly:", toxic)
 		}
 		toxic = AssertToxicExists(t, toxics, "latency_downstream", "latency", "downstream", true)
-		if toxic.Toxicity != 1.0 || toxic.Attributes["latency"] != 0.0 || toxic.Attributes["jitter"] != 0.0 {
+		if toxic.Toxicity != 1.0 || toxic.Attributes["latency"] != 0.0 ||
+			toxic.Attributes["jitter"] != 0.0 {
 			t.Fatal("Toxic was not read back correctly:", toxic)
 		}
 	})
@@ -537,7 +953,8 @@ func TestAddToxicWithToxicity(t *testing.T) {
 			t.Fatal("Error setting toxic:", err)
 		}
 
-		if latency.Toxicity != 0.2 || latency.Attributes["latency"] != 100.0 || latency.Attributes["jitter"] != 10.0 {
+		if latency.Toxicity != 0.2 || latency.Attributes["latency"] != 100.0 ||
+			latency.Attributes["jitter"] != 10.0 {
 			t.Fatal("Latency toxic did not start up with correct settings:", latency)
 		}
 
@@ -546,7 +963,8 @@ func TestAddToxicWithToxicity(t *testing.T) {
 			t.Fatal("Error returning toxics:", err)
 		}
 		toxic := AssertToxicExists(t, toxics, "latency_downstream", "latency", "downstream", true)
-		if toxic.Toxicity != 0.2 || toxic.Attributes["latency"] != 100.0 || toxic.Attributes["jitter"] != 10.0 {
+		if toxic.Toxicity != 0.2 || toxic.Attributes["latency"] != 100.0 ||
+			toxic.Attributes["jitter"] != 10.0 {
 			t.Fatal("Toxic was not read back correctly:", toxic)
 		}
 	})
@@ -564,7 +982,8 @@ func TestAddNoop(t *testing.T) {
 			t.Fatal("Error setting toxic:", err)
 		}
 
-		if noop.Toxicity != 1.0 || noop.Name != "foobar" || noop.Type != "noop" || noop.Stream != "downstream" {
+		if noop.Toxicity != 1.0 || noop.Name != "foobar" || noop.Type != "noop" ||
+			noop.Stream != "downstream" {
 			t.Fatal("Noop toxic did not start up with correct settings:", noop)
 		}
 
@@ -594,7 +1013,8 @@ func TestUpdateToxics(t *testing.T) {
 			t.Fatal("Error setting toxic:", err)
 		}
 
-		if latency.Toxicity != 1.0 || latency.Attributes["latency"] != 100.0 || latency.Attributes["jitter"] != 10.0 {
+		if latency.Toxicity != 1.0 || latency.Attributes["latency"] != 100.0 ||
+			latency.Attributes["jitter"] != 10.0 {
 			t.Fatal("Latency toxic did not start up with correct settings:", latency)
 		}
 
@@ -605,7 +1025,8 @@ func TestUpdateToxics(t *testing.T) {
 			t.Fatal("Error setting toxic:", err)
 		}
 
-		if latency.Toxicity != 0.5 || latency.Attributes["latency"] != 1000.0 || latency.Attributes["jitter"] != 10.0 {
+		if latency.Toxicity != 0.5 || latency.Attributes["latency"] != 1000.0 ||
+			latency.Attributes["jitter"] != 10.0 {
 			t.Fatal("Latency toxic did not get updated with the correct settings:", latency)
 		}
 
@@ -616,7 +1037,8 @@ func TestUpdateToxics(t *testing.T) {
 			t.Fatal("Error setting toxic:", err)
 		}
 
-		if latency.Toxicity != 0.5 || latency.Attributes["latency"] != 500.0 || latency.Attributes["jitter"] != 10.0 {
+		if latency.Toxicity != 0.5 || latency.Attributes["latency"] != 500.0 ||
+			latency.Attributes["jitter"] != 10.0 {
 			t.Fatal("Latency toxic did not get updated with the correct settings:", latency)
 		}
 
@@ -626,7 +1048,8 @@ func TestUpdateToxics(t *testing.T) {
 		}
 
 		toxic := AssertToxicExists(t, toxics, "latency_downstream", "latency", "downstream", true)
-		if toxic.Toxicity != 0.5 || toxic.Attributes["latency"] != 500.0 || toxic.Attributes["jitter"] != 10.0 {
+		if toxic.Toxicity != 0.5 || toxic.Attributes["latency"] != 500.0 ||
+			toxic.Attributes["jitter"] != 10.0 {
 			t.Fatal("Toxic was not read back correctly:", toxic)
 		}
 	})
@@ -650,7 +1073,8 @@ func TestRemoveToxic(t *testing.T) {
 		}
 
 		toxic := AssertToxicExists(t, toxics, "latency_downstream", "latency", "downstream", true)
-		if toxic.Toxicity != 1.0 || toxic.Attributes["latency"] != 0.0 || toxic.Attributes["jitter"] != 0.0 {
+		if toxic.Toxicity != 1.0 || toxic.Attributes["latency"] != 0.0 ||
+			toxic.Attributes["jitter"] != 0.0 {
 			t.Fatal("Toxic was not read back correctly:", toxic)
 		}
 
@@ -673,13 +1097,14 @@ func TestVersionEndpointReturnsVersion(t *testing.T) {
 		if err != nil {
 			t.Fatal("Failed to get index", err)
 		}
+		defer resp.Body.Close()
 
-		body, err := ioutil.ReadAll(resp.Body)
+		body, err := io.ReadAll(resp.Body)
 		if err != nil {
 			t.Fatal("Unable to read body from response")
 		}
 
-		if string(body) != toxiproxy.Version {
+		if string(body) != "{\"version\": \"git\"}\n" {
 			t.Fatal("Expected to return Version from /version, got:", string(body))
 		}
 	})
@@ -699,13 +1124,18 @@ func TestInvalidStream(t *testing.T) {
 	})
 }
 
-func AssertToxicExists(t *testing.T, toxics tclient.Toxics, name, typeName, stream string, exists bool) *tclient.Toxic {
+func AssertToxicExists(
+	t *testing.T,
+	toxics tclient.Toxics,
+	name, typeName, stream string,
+	exists bool,
+) *tclient.Toxic {
 	var toxic *tclient.Toxic
 	var actualType, actualStream string
 
-	for _, tox := range toxics {
+	for i, tox := range toxics {
 		if name == tox.Name {
-			toxic = &tox
+			toxic = &toxics[i]
 			actualType = tox.Type
 			actualStream = tox.Stream
 		}
@@ -713,9 +1143,13 @@ func AssertToxicExists(t *testing.T, toxics tclient.Toxics, name, typeName, stre
 	if exists {
 		if toxic == nil {
 			t.Fatalf("Expected to see %s toxic in list", name)
-		} else if actualType != typeName {
+		}
+
+		if actualType != typeName {
 			t.Fatalf("Expected %s to be of type %s, found %s", name, typeName, actualType)
-		} else if actualStream != stream {
+		}
+
+		if actualStream != stream {
 			t.Fatalf("Expected %s to be in stream %s, found %s", name, stream, actualStream)
 		}
 	} else if toxic != nil && actualStream == stream {
